package com.apl;

import java.io.BufferedReader;
import java.io.File;
import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.InputStreamReader;
import java.util.ArrayList;
import java.util.List;

import com.bmc.arsys.api.ARException;
import com.bmc.arsys.api.AttachmentValue;
import com.bmc.arsys.api.Constants;
import com.bmc.arsys.api.DataType;
import com.bmc.arsys.api.StatusInfo;
import com.bmc.arsys.api.Value;
import com.bmc.arsys.pluginsvr.plugins.ARFilterAPIPlugin;
import com.bmc.arsys.pluginsvr.plugins.ARPluginContext;

public class AttachmentValidation extends ARFilterAPIPlugin {
	private final static String app = "APL Attachment Validation";
	private final static String ver = "1.3";

	private boolean virusCheck = false;
	private String virusProgram, virusParams, outputDir, warningText, errorText;
	private int errorNumber = 525;

	@Override
	public void initialize(ARPluginContext context){
		context.logMessage(context.getPluginInfo(), ARPluginContext.PLUGIN_LOG_LEVEL_INFO, "Initalizing " + app + " v" + ver);

		String configVirusCheck = context.getConfigItem("virusCheck");
		if (configVirusCheck != null && !configVirusCheck.isEmpty() && configVirusCheck.equalsIgnoreCase("true")) {
			virusCheck = true;
		}

		String configVirusProgram = context.getConfigItem("virusProgram");
		if (configVirusProgram != null && !configVirusProgram.isEmpty()) {
			virusProgram = configVirusProgram;
		}

		String configVirusParams = context.getConfigItem("virusParams");
		if (configVirusParams != null && !configVirusParams.isEmpty()) {
			virusParams = configVirusParams;
		}

		String configOutputDir = context.getConfigItem("outputDir");
		if (configOutputDir != null && !configOutputDir.isEmpty()) {
			outputDir = configOutputDir;
		}

		String configWarningText = context.getConfigItem("warningText");
		if (configWarningText != null && !configWarningText.isEmpty()) {
			warningText = configWarningText;
		}

		String configErrorText = context.getConfigItem("errorText");
		if (configErrorText != null && !configErrorText.isEmpty()) {
			errorText = configErrorText;
		}

		String configErrorNumber = context.getConfigItem("errorNumber");
		if (configErrorNumber != null && !configErrorNumber.isEmpty()) {
			errorNumber = Integer.parseInt(configErrorNumber);
		}
	}

	@Override
	public void onEvent(ARPluginContext context, int eventId) throws ARException {
		super.onEvent(context, eventId);
		initialize(context);
	}

	/*
	 * This is the main filter api call that is passed a list of values that the filter action has setup
	 * as input values.  A list of Value objects is expected as an output.
	 */
	@Override
	public List<Value> filterAPICall(ARPluginContext context, List<Value> inValueList) throws ARException{
		context.logMessage(context.getPluginInfo(), ARPluginContext.PLUGIN_LOG_LEVEL_INFO, "filterapiCall()");

		ArrayList<Value> outValueList = new ArrayList<Value>();

		Value inValue = null;
		try {
			inValue = inValueList.get(0);
		} catch (Exception e) {
			throwError(context, "Error getting attachment: " + e.getMessage());
		}

		if (inValue != null && inValue.getDataType() == DataType.ATTACHMENT) {
			AttachmentValue attValue = (AttachmentValue)inValue.getValue();
			byte[] byteArray = attValue.getContent();

			File outputDirFile = new File(outputDir);
			File outputFile = null;

			if (!outputDirFile.exists()) {
				throwError(context, "Specified directory " + outputDir + " does not exist");
			} else if (!outputDirFile.isDirectory()) {
				throwError(context, "Specified directory " + outputDir + " is not a directory");
			} else if (!outputDirFile.canWrite()) {
				throwError(context, "Specified directory " + outputDir + " is not writable");
			}

			String fileName = null;
			fileName = attValue.getValueFileName();
			if (fileName.contains(File.separator))
				fileName = fileName.substring(fileName.lastIndexOf(File.separator)+1);

			outputFile = new File(outputDirFile.getAbsoluteFile() + File.separator + fileName);
			FileOutputStream fos;
			try {
				context.logMessage(context.getPluginInfo(), ARPluginContext.PLUGIN_LOG_LEVEL_INFO, "Saving to " + outputFile.getAbsolutePath());
				fos = new FileOutputStream(outputFile);
				fos.write(byteArray);
				fos.close();    		
			} catch (FileNotFoundException e) {
				throwError(context, "FileNotFoundException: " + e.getMessage());
			} catch (IOException e) {
				throwError(context, "IOException: " + e.getMessage());
			}

			if (virusCheck) {
				String curVirusParams = virusParams;
				File virusScanFile = new File(virusProgram);

				if (!virusScanFile.exists()) {
					throwError(context, "Specified file " + virusProgram + " does not exist");
				} else if (!virusScanFile.isFile()) {
					throwError(context, "Specified file " + virusProgram + " is not a file");
				} else if (!virusScanFile.canExecute()) {
					throwError(context, "Specified file " + virusProgram + " is not executable");
				} else if (curVirusParams.contains("$FILENAME$")) {
					curVirusParams = curVirusParams.replace("$FILENAME$", outputFile.getAbsolutePath());
				}

				Process process = null;
				try {
					List<String> commandList = new ArrayList<String>();
					commandList.add(virusProgram);
					commandList.add(curVirusParams);

					ProcessBuilder processBuilder = new ProcessBuilder(commandList);
					processBuilder.redirectErrorStream(true);

					//context.logMessage(context.getPluginInfo(), ARPluginContext.PLUGIN_LOG_LEVEL_INFO, "Executing command: " + processBuilder.toString());

					process = processBuilder.start();
				} catch (IOException e) {
					throwError(context, "IOException starting process: " + e.getMessage());
				}

				StringBuilder sb = new StringBuilder();
				int processExitCode = 0;
				if (process != null) {
					BufferedReader stdOut = new BufferedReader(new InputStreamReader(process.getInputStream()));
					String line = null;

					while (process.isAlive()) {
						try {
							line = stdOut.readLine();
							if (line != null) {
								//context.logMessage(context.getPluginInfo(), ARPluginContext.PLUGIN_LOG_LEVEL_INFO, "Appending: " + line);
								sb.append(line + "\n");
							}
						} catch (IOException e) {
							throwError(context, "IOException getting output line: " + e.getMessage());
						}
					}
					processExitCode = process.exitValue();
				}

				String outputText = sb.toString();
				context.logMessage(context.getPluginInfo(), ARPluginContext.PLUGIN_LOG_LEVEL_INFO, "Command Exit Code: " + processExitCode);
				context.logMessage(context.getPluginInfo(), ARPluginContext.PLUGIN_LOG_LEVEL_INFO, "Command Output: " + outputText);
				if (processExitCode > 0 || (outputText != null && !outputText.isEmpty() && errorText != null && !errorText.isEmpty() && outputText.contains(errorText))) {
					throwError(context, outputText);
				} else if (outputText != null && !outputText.isEmpty()  && warningText != null && !warningText.isEmpty() && outputText.contains(warningText)) {
					Value warningValue = new Value(outputFile.getAbsolutePath());
					outValueList.add(warningValue);
					context.logMessage(context.getPluginInfo(), ARPluginContext.PLUGIN_LOG_LEVEL_WARNING, "Warning: " + outputText);
				}
			}

			if (outputFile != null && outputFile.exists()) {
				boolean success = outputFile.delete();
				if (!success)
					context.logMessage(context.getPluginInfo(), ARPluginContext.PLUGIN_LOG_LEVEL_ERROR, "Error removing attachment " + outputFile.getAbsolutePath());
			}
		} else {
			throwError(context, "Error: Input DataType was not attachment");
		}

		return outValueList;
	}

	private void throwError(ARPluginContext context, String errorMessage) throws ARException {
		context.logMessage(context.getPluginInfo(), ARPluginContext.PLUGIN_LOG_LEVEL_ERROR, "Error: " + errorMessage);
		StatusInfo si = new StatusInfo();
		si.setMessageText(errorMessage);
		si.setMessageType(Constants.AR_RETURN_ERROR);
		si.setMessageNum(errorNumber);
		List<StatusInfo> siList = new ArrayList<StatusInfo>();
		siList.add(si);
		throw new ARException(siList);
	}
}
